import React, { useEffect, useMemo, useRef, useState } from 'react';
import { Table, AugmentedNodeAttributes } from './components/Table';
import { SchemaAttribute } from '../../../schema';
import { VisualizationPropTypes, VISComponentType, VisualizationSettingsPropTypes } from '../../common';
import { Input } from '@graphpolaris/shared/lib/components/inputs';
import { SettingsContainer } from '@graphpolaris/shared/lib/vis/components/config';
import { Button } from '@graphpolaris/shared/lib/components/buttons';
import { ArrowDropDown } from '@mui/icons-material';

export type TableProps = {
  id: string;
  name: string;
  showBarplot: boolean;
  itemsPerPage: number;
  displayAttributes: string[];
  displayEntity: string;
  maxBarsCount: number;
};

const settings: TableProps = {
  id: 'TableVis',
  name: 'TableVis',
  itemsPerPage: 10,
  showBarplot: true,
  displayAttributes: [],
  displayEntity: '',
  maxBarsCount: 10,
};

export const TableVis = ({ data, schema, settings, updateSettings, graphMetadata }: VisualizationPropTypes<TableProps>) => {
  const ref = useRef<HTMLDivElement>(null);

  useEffect(() => {
    if (!graphMetadata.nodes.labels.includes(settings.displayEntity)) {
      updateSettings({ displayEntity: graphMetadata.nodes.labels[0] });
    }
  }, [graphMetadata.nodes.labels, data, settings]);

  const attributesArray = useMemo<AugmentedNodeAttributes[]>(
    () =>
      data.nodes
        .filter((node) => node.label === settings.displayEntity)
        .map((node) => {
          const types: SchemaAttribute[] =
            schema.nodes.find((n) => n.key === node.label)?.attributes?.attributes ??
            schema.edges.find((r) => r.key === node.label)?.attributes?.attributes ??
            [];

          return {
            attribute: node.attributes,
            type: Object.fromEntries(types.map((t) => [t.name, t.type])),
          };
        }),
    [data.nodes, settings.displayEntity],
  );

  return (
    <div className="h-full w-full" ref={ref}>
      {attributesArray.length > 0 && (
        <Table
          data={attributesArray}
          itemsPerPage={settings.itemsPerPage}
          showBarPlot={settings.showBarplot}
          showAttributes={settings.displayAttributes}
          selectedEntity={settings.displayEntity}
          maxBarsCount={settings.maxBarsCount}
        />
      )}
    </div>
  );
};

const TableSettings = ({ settings, graphMetadata, updateSettings }: VisualizationSettingsPropTypes<TableProps>) => {
  useEffect(() => {
    if (graphMetadata && graphMetadata.nodes && graphMetadata.nodes.labels.length > 0) {
      updateSettings({ displayEntity: graphMetadata.nodes.labels[0] });
    }
  }, [graphMetadata]);

  const [areCollapsedAttr, setAreCollapsedAttr] = useState<boolean>(true);
  const toggleCollapseAttr = () => {
    setAreCollapsedAttr(!areCollapsedAttr);
  };
  const selectedNodeAttributes = useMemo(() => {
    if (settings.displayEntity) {
      const nodeType = graphMetadata.nodes.types[settings.displayEntity];
      if (nodeType && nodeType.attributes) {
        return Object.keys(nodeType.attributes);
      }
    }
    return [];
  }, [settings.displayEntity, graphMetadata]);

  useEffect(() => {
    if (graphMetadata && graphMetadata.nodes && graphMetadata.nodes.labels.length > 0) {
      updateSettings({ displayAttributes: selectedNodeAttributes });
    }
  }, [selectedNodeAttributes]);

  return (
    <SettingsContainer>
      <Input
        type="dropdown"
        label="Select entity"
        value={settings.displayEntity}
        onChange={(val) => updateSettings({ displayEntity: val as string })}
        options={graphMetadata.nodes.labels}
      />
      <Input type="boolean" label="Show barplot" value={settings.showBarplot} onChange={(val) => updateSettings({ showBarplot: val })} />
      <Input
        type="dropdown"
        label="Items per page"
        value={settings.itemsPerPage}
        onChange={(val) => updateSettings({ itemsPerPage: val as number })}
        options={[10, 25, 50, 100]}
      />
      <Input
        type="number"
        label="Max Bars in Bar Plots"
        value={settings.maxBarsCount}
        onChange={(val) => updateSettings({ maxBarsCount: val })}
      />
      <div>
        <span className="text-sm">Attributes to display:</span>
        <Button
          className="w-full text-justify justify-start"
          variantType="secondary"
          variant="ghost"
          size="sm"
          onClick={toggleCollapseAttr}
          iconComponent={<ArrowDropDown />}
        >
          attributes:{' '}
        </Button>
        <div className="">
          {!areCollapsedAttr && (
            <Input
              type="checkbox"
              value={settings.displayAttributes}
              options={selectedNodeAttributes}
              onChange={(val: string[] | string) => {
                const updatedVal = Array.isArray(val) ? val : [val];
                updateSettings({ displayAttributes: updatedVal });
              }}
            />
          )}
        </div>
      </div>
    </SettingsContainer>
  );
};

export const TableComponent: VISComponentType<TableProps> = {
  displayName: 'TableVis',
  component: TableVis,
  settingsComponent: TableSettings,
  settings: settings,
};

export default TableComponent;
